<?php

class NewsletterElasticEmail extends NewsletterMailerAddon {

    /**
     * @var NewsletterElasticEmail
     */
    static $instance;

    function __construct($version) {
        self::$instance = $this;
        $this->menu_title = 'Elastic Email';
        parent::__construct('elasticemail', $version, __DIR__);
    }

    function get_mailer() {
        static $mailer = null;
        if (!$mailer) {
            $mailer = new NewsletterElasticEmailMailer($this->name, $this->options);
        }
        return $mailer;
    }

    function init() {
        parent::init();
        add_action('newsletter_elasticemail_bounce', array($this, 'bounce'));

        // Earlier than unsubscription module
        if ($this->enabled) {
            add_action('newsletter_replace', array($this, 'hook_newsletter_replace'), 5, 3);
        }

        if (is_admin()) {
            if (!defined('DOING_CRON') || !DOING_CRON) {
                if (wp_get_schedule('newsletter_elasticemail_bounce') === false) {
                    wp_schedule_event(time() + 60, 'daily', 'newsletter_elasticemail_bounce');
                }
            }
        }

        add_action('newsletter_user_deleted', array($this, 'hook_newsletter_user_deleted'));
        add_action('newsletter_user_confirmed', array($this, 'hook_newsletter_user_deleted'));
        add_action('newsletter_user_reactivated', array($this, 'hook_newsletter_user_deleted'));
    }

    function weekly_check() {
        parent::weekly_check();
        $license_key = Newsletter::instance()->get_license_key();
        $response = wp_remote_post('https://www.thenewsletterplugin.com/wp-content/addon-check.php?k=' . urlencode($license_key)
                . '&a=' . urlencode($this->name) . '&d=' . urlencode(home_url()) . '&v=' . urlencode($this->version)
                . '&ml=' . (Newsletter::instance()->is_multilanguage() ? '1' : '0'));
    }

    function hook_newsletter_replace($text, $user, $email) {
        $text = str_replace('{unsubscription_url}', '{unsubscribe:{unsubscription_url}}', $text);
        $text = str_replace('{unsubscription_confirm_url}', '{unsubscribe:{unsubscription_confirm_url}}', $text);
        return $text;
    }

    /**
     *
     * @param TNP_User $user
     * @return type
     */
    function hook_newsletter_user_deleted($user) {
        $logger = $this->get_logger();
        $this->logger->debug('Deleting user ' . $user->email);
        $data['apikey'] = $this->options['api_key'];
        $data['emails'] = $user->email;
        $response = wp_remote_post("https://api.elasticemail.com/v2//contact/delete", array('body' => $data));
        if (is_wp_error($response)) {
            $this->logger->error($response);
            return;
        }
        $result = json_decode(wp_remote_retrieve_body($response));
        // Returns ok even if the contact is not there, and that's good
        if ($result->success === false) {
            $this->logger->error($result);
            return;
        }
        $this->logger->debug($result);
    }

    /**
     *
     * @global wpdb $wpdb
     */
    var $bounces_result = null;
    var $complaints_result = null;
    var $unsubscribes_result = null;
    var $bounce_emails = [];
    var $unsubscribed_emails = [];
    var $complained_emails = [];

    function bounce() {
        global $wpdb, $newsletter;
        $logger = $this->get_logger();

        $list = $this->get_suppressions('bounces');

        if (is_wp_error($list)) {
            $this->bounces_result = $list;
        } else {
            foreach ($list as $item) {
                $email = strtolower($item->Email);
                $status = 'B';
                $this->bounce_emails[] = $email;

                $wpdb->query($wpdb->prepare("update " . NEWSLETTER_USERS_TABLE . " set status=%s where email=%s limit 1", $status, $email));
            }
        }


        $list = $this->get_suppressions('unsubscribes');

        if (is_wp_error($list)) {
            $this->unsubscribes_result = $list;
        } else {
            foreach ($list as $item) {
                $email = strtolower($item->Email);
                $status = 'U';
                $this->$unsubscribed_emails[] = $email;

                $wpdb->query($wpdb->prepare("update " . NEWSLETTER_USERS_TABLE . " set status=%s where email=%s limit 1", $status, $email));
            }
        }

        $list = $this->get_suppressions('complaints');

        if (is_wp_error($list)) {
            $this->complaints_result = $list;
        } else {
            foreach ($list as $item) {
                $email = strtolower($item->Email);
                $status = 'P';
                $this->$complained_emails[] = $email;

                $wpdb->query($wpdb->prepare("update " . NEWSLETTER_USERS_TABLE . " set status=%s where email=%s limit 1", $status, $email));
            }
        }


        $this->save_last_run(time());
    }

    function get_suppressions($type) {
        $logger = $this->get_logger();

        $data['apikey'] = $this->options['api_key'];
        //$data['statuses'] = 'Bounced,Unsubscribed';
        //$data['search'] = '';
        $data['limit'] = 1000;

        // https://elasticemail.com/developers/api-documentation/rest-api#tag/Suppressions
        $response = wp_remote_get("https://api.elasticemail.com/v4/suppressions/" . $type, ['body' => $data]);

        if (is_wp_error($response)) {
            /* @var $response WP_Error */
            $logger->error($response);
            return $response;
        }

        $http_code = wp_remote_retrieve_response_code($response);
        $http_body = wp_remote_retrieve_body($response);
        if ($http_code != 200) {
            //die();
            $error = new WP_Error('1', 'HTTP Error ' . $http_code . '<br>' . $http_body);
            $logger->error($error);
            return $error;
        }

        $list = json_decode($http_body);

        if ($list === false) {
            $error = new WP_Error('1', 'Unable to decode the JSON response: ' . $http_body);
            $logger->error($error);
            $logger->error($http_body);
            return $error;
        }

        return $list;
    }
}

class NewsletterElasticEmailMailer extends NewsletterMailer {

    function __construct($name, $options = array()) {
        parent::__construct($name, $options);
        if (!empty($this->options['turbo'])) {
            $this->batch_size = (int) $this->options['turbo'];
        }
    }

    function get_description() {
        return 'Elastic Email';
    }

    /**
     *
     * @param TNP_Mailer_Message $message
     * @return array
     */
    function build_data($message) {
        $logger = $this->get_logger();
        $newsletter = Newsletter::instance();

        $data = [];

        $data['subject'] = $message->subject;
        $data['from'] = $newsletter->options['sender_email'];
        $data['fromName'] = $newsletter->options['sender_name'];

        $data['replyTo'] = $newsletter->options['reply_to'];
        $data['to'] = $message->to;

        $i = 0;
        foreach ($message->headers as $key => $value) {
            // Be aware of the space after the ":", the key "header_[something]" is not an error!
            $data['headers_' . $i] = $key . ': ' . $value;
        }

        $data['bodyHtml'] = $message->body;
        if (empty($message->body) || empty($this->options['disable_text_part'])) {
            $data['bodyText'] = $message->body_text;
        }


        $data['apikey'] = $this->options['api_key'];

        return $data;
    }

    /**
     *
     * @param TNP_Mailer_Message $message
     * @return type
     */
    function build_curl($message) {
        $curl = curl_init('https://api.elasticemail.com/v2/email/send');
        curl_setopt($curl, CURLOPT_POST, true);
        $data = $this->build_data($message);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
        curl_setopt($curl, CURLOPT_TIMEOUT, 60);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        // Tell curl not to return headers, but do return the response
        curl_setopt($curl, CURLOPT_HEADER, false);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        return $curl;
    }

    /**
     *
     * @param TNP_Mailer_Message $message
     * @return \WP_Error|boolean
     */
    function send($message) {
        $logger = $this->get_logger();
        $newsletter = Newsletter::instance();

        $curl = $this->build_curl($message);

        $response = curl_exec($curl);

        if (curl_errno($curl) != 0) {
            $message->error = curl_errno($curl) . ' - ' . curl_error($curl);
        }

        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);

        if ($code < 200 || $code > 299) {
            $message->error = 'Http error ' . $code;
        }

        $jsonResult = json_decode($response);

        if ($jsonResult === false) {
            $message->error = 'Unable to decode the response: ' . $response;
        }

        if ($jsonResult->success === false) {
            $message->error = $jsonResult->error;
        }

        curl_close($curl);

        if ($message->error) {
            $wp_error = new WP_Error(self::ERROR_GENERIC, $message->error);
            $logger->error($wp_error);
            return $wp_error;
        }
        return true;
    }

    public function send_chunk($messages) {

        $logger = $this->get_logger();
        $mh = curl_multi_init();

        foreach ($messages as $message) {
            $ch = $this->build_curl($message);
            $message->ch = $ch;
            curl_multi_add_handle($mh, $ch);
        }
        $active = 1;
        $start = time();
        while ($active) {
            curl_multi_exec($mh, $active);
            curl_multi_select($mh);
            if (time() - $start > 300) {
                break;
            }
        }

        $wp_error = null;
        foreach ($messages as $message) {
            $curl = $message->ch;
            $errno = curl_errno($curl);
            $logger->debug('cUrl error number: ' . $errno);

            if ($errno != 0) {
                $message->error = 'cURL error: ' . curl_errno($curl) . ' - ' . curl_error($curl);
                $wp_error = new WP_Error(self::ERROR_GENERIC, $message->error);
            } else {

                $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);

                $response = curl_multi_getcontent($curl);

                if ($code < 200 || $code > 299) {
                    $message->error = 'HTTP Error ' . $code . ' - ' . $response;
                    $wp_error = new WP_Error(self::ERROR_GENERIC, $message->error);
                } else {
                    $jsonResult = json_decode($response);

                    if ($jsonResult === false) {
                        $message->error = 'Unable to decode the response: ' . $response;
                        $wp_error = new WP_Error(self::ERROR_GENERIC, $message->error);
                    }

                    if ($jsonResult->success === false) {
                        $message->error = $jsonResult->error;
                        $wp_error = new WP_Error(self::ERROR_GENERIC, $message->error);
                    }
                }
            }
        }

        curl_multi_close($mh);

        if ($wp_error) {
            $logger->error($wp_error);
            return $wp_error;
        }
        return true;
    }
}
